<?php

namespace Drupal\editoria11y_si;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\State\StateInterface;
use Drupal\key\KeyRepositoryInterface;
use GuzzleHttp\Exception\RequestException;

/**
 * SiteImprove data importer.
 *
 * Handles importing SiteImprove data from SiteImprove API.
 */
class Editoria11ySiImporter {

  /**
   * Email address of SiteImprove user this is tied to.
   *
   * @var string
   */
  private string $apiUser;

  /**
   * API key of the user.
   *
   * @var string
   */
  private string $apiKey;

  /**
   * SiteImprove site that you want to reference.
   *
   * @var string
   */
  private string $site;

  /**
   * The website group for the APIs.
   *
   * @var string
   */
  private string $group;

  /**
   * The website's domain that will be stripped from results.
   *
   * @var string
   */
  private string $domain;

  /**
   * Constructs a new connection object.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   The logger factory instance.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Database\Connection $connection
   *   The connection for database connection objects.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger for message objects.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager interface.
   * @param \Drupal\Core\State\StateInterface $state
   *   The State Interface.
   * @param \Drupal\key\KeyRepositoryInterface $keyRepository
   *   The Key repository interface.
   * @param \Drupal\Core\Queue\QueueFactory $queueFactory
   *   Drupal queue factory instance.
   */
  public function __construct(
    // @todo use loggerFactory or get rid of it.
    private LoggerChannelFactoryInterface $loggerFactory,
    private ConfigFactoryInterface $configFactory,
    private Connection $connection,
    private MessengerInterface $messenger,
    private EntityTypeManagerInterface $entityTypeManager,
    private StateInterface $state,
    private KeyRepositoryInterface $keyRepository,
    private readonly QueueFactory $queueFactory,
  ) {}

  /**
   * Helper function to get the secrets and other init items.
   */
  private function loadConnectionInformation() {
    // Load the settings from /admin/config/content/editoria11y/si.
    $config = $this->configFactory->get('editoria11y_si.settings');
    $this->domain = $config->get('domain');
    $key_id = $config->get('siteimprove_api');
    $keys = $this->keyRepository->getKey($key_id);

    $this->apiUser = $keys->getKeyValues()['username'] ?? '';
    $this->apiKey = $keys->getKeyValues()['api_key'] ?? '';
    $this->site = $keys->getKeyValues()['site'] ?? '';
    $this->group = $keys->getKeyValues()['group'] ?? '';
  }

  /**
   * Helper function to get the pages from the API.
   *
   * @param string $url
   *   The url of the api to get data from.
   */
  private function getPages(string $url) {
    // Get all of the pages.
    try {

      $process = curl_init($url);
      curl_setopt($process, CURLOPT_HTTPHEADER, ['Accept: application/json']);
      curl_setopt($process, CURLOPT_USERPWD, $this->apiUser . ":" . $this->apiKey);
      curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);

      $response = curl_exec($process);
      // $status = curl_getinfo($process);
      curl_close($process);
      $data = json_decode($response, TRUE);
      return $data;
    }
    catch (RequestException $e) {
      // An error happened.
    }
  }

  /**
   * Generate editoria11y_si import queue items.
   */
  public function generateImportQueueItems() {
    $this->loadConnectionInformation();

    $pageNumber = 1;
    $url = "https://api.siteimprove.com/v2/sites/" . $this->site . "/quality_assurance/links/pages_with_broken_links?page=" . $pageNumber . "&page_size=100";
    if (!empty($this->group)) {
      $url .= "&group_id=" . $this->group;
    }

    // Loop through all of the pages of siteImprove pages.
    $pagesWithBrokenLinks = [];
    $pageUrls = [];
    do {
      $data = $this->getPages($url);
      // Each $data will have 100 items. Need to extract the info in this.
      foreach ($data['items'] as $page) {

        // Loop through all urls and get their information.
        $pageUrl = "https://api.siteimprove.com/v2/sites/" . $this->site . "/quality_assurance/links/pages_with_broken_links/" . $page['id'] . "/broken_links";
        $process = curl_init($pageUrl);
        curl_setopt($process, CURLOPT_HTTPHEADER, ['Accept: application/json']);
        curl_setopt($process, CURLOPT_USERPWD, $this->apiUser . ":" . $this->apiKey);
        curl_setopt($process, CURLOPT_RETURNTRANSFER, TRUE);

        $response = curl_exec($process);
        // $status = curl_getinfo($process);
        curl_close($process);
        $pageData = json_decode($response, TRUE);
        // Removing the domain from the url since internal urls often
        // do not have this.
        $page['url'] = str_replace($this->domain, '', $page['url']);
        $pagesWithBrokenLinks[] = [
          'url' => $page['url'] ?? '',
          'siteimprove_url' => $page['_siteimprove']['page_report']['href'] ?? '',
          'raw_data' => $pageData['items'] ?? [],
        ];
        $pageUrls[] = $page['url'] ?? '';
      }
      // Check to see if there is a next page of links. If not, we are done.
      $url = $data['links']['next']['href'] ?? FALSE;
    } while ($url);

    if (!empty($pageUrls)) {
      // Get ids of all editoria11y_si entities that are no longer in the API.
      $editoria11ySiEntitiesToDeleteQuery = $this->entityTypeManager->getStorage('editoria11y_si')
        ->getQuery()
        ->accessCheck(FALSE)
        ->condition('siteimprove_type', 'broken_links')
        ->condition('url', $pageUrls, 'NOT IN');
      $editoria11ySiEntitiesToDeleteIds = $editoria11ySiEntitiesToDeleteQuery->execute();

      // Load all the editoria11y_si we found and delete them.
      $editoria11ySiEntitiesToDelete = $this->entityTypeManager->getStorage('editoria11y_si')->loadMultiple($editoria11ySiEntitiesToDeleteIds);
      $this->entityTypeManager->getStorage('editoria11y_si')->delete($editoria11ySiEntitiesToDelete);

      // Get the editoria11y_si broken links import queue.
      $editoria11ySiBrokenLinksImportQueue = $this->queueFactory->get('editoria11y_si_broken_links_import_processor');
      // Delete any left over items from the queue.
      $editoria11ySiBrokenLinksImportQueue->deleteQueue();

      // Create a queue item for each siteimprove page.
      foreach ($pagesWithBrokenLinks as $pagesWithBrokenLinksItem) {
        $editoria11ySiBrokenLinksImportQueue->createItem($pagesWithBrokenLinksItem);
      }
    }

  }

}
