<?php

declare(strict_types=1);

namespace Drupal\editoria11y_si\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\State\State;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Configure editoria11y_si settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * Editoria11y SI SettingsForm construct.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Config factory instance.
   * @param \Drupal\Core\State\State $state
   *   Drupal state system object.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request
   *   The request object.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    private readonly State $state,
    private readonly RequestStack $request,
  ) {
    parent::__construct($configFactory);
  }

  /**
   * {@inheritdoc}
   */
  #[\Override]
  public static function create(ContainerInterface $container) {
    $form = new static(
      $container->get('config.factory'),
      $container->get('state'),
      $container->get('request_stack'),
    );
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'editoria11y_si_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['editoria11y_si.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('editoria11y_si.settings');
    $form['overview'] = [
      '#type' => 'markup',
      '#markup' => 'If you NEED to delete ALL editoria11y_si, drush entity:delete editoria11y_si</p><p>See the reports page at <a href="/admin/reports/editoria11y/si-broken-links">/admin/reports/editoria11y/si-broken-links</a> to see all broken links.</p>',
    ];
    $form['siteimprove_api'] = [
      '#title' => $this->t("SiteImprove API key"),
      '#type' => 'key_select',
      '#key_filters' => ['type' => 'editoria11y_si_key'],
      '#default_value' => $config->get('siteimprove_api'),
      '#required' => TRUE,
    ];

    $domain = $this->request->getCurrentRequest()->getSchemeAndHttpHost();
    $form['domain'] = [
      '#title' => $this->t("Domain"),
      '#type' => 'textfield',
      '#default_value' => $config->get('domain') ?? $domain,
      '#description' => $this->t('Domain of website. Will strip this from SiteImprove data since internal urls often do not have this.'),
      '#required' => TRUE,
    ];

    $lastImported = $this->state->get('editoria11y_si.import.last_update');
    $form['manual_import_broken_links'] = [
      '#type' => 'submit',
      '#value' => $this->t('Update SiteImprove broken link data manually'),
      '#description' => $this->t('Force the data to update from SiteImprove.'),
      '#submit' => ['::startImportOfBrokenLinks'],
      '#prefix' => $this->t(
      "<div class='description'>Last successful import: @time</div>",
        ['@time' => $lastImported]
      ),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->config('editoria11y_si.settings')
      ->set('siteimprove_api', $form_state->getValue('siteimprove_api'))
      ->set('domain', $form_state->getValue('domain'))
      ->save();
    parent::submitForm($form, $form_state);
  }

  /**
   * Manual import Site Improve pages.
   */
  public function startImportOfBrokenLinks(array &$form, FormStateInterface $form_state) {
    editoria11y_si_queue_cronjob();
    $message = $this->t('SiteImprove broken link data imported.');
    $this->messenger()->addStatus($message);
  }

}
